#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

import re
import ctypes
import ctypes.wintypes
import comtypes
import comtypes.automation
import comtypes.client
import sys

from ..utils.win_utils import WinUtils
from .. import TooSaltyUISoupException
from ..interfaces.i_soup import ISoup
from .element import WinElement
from .mouse import WinMouse
from .keyboard import WinKeyboard

if WinUtils.is_python_3():
    basestring = unicode = str

comtypes.client.GetModule('oleacc.dll')


class WinSoup(ISoup):

    mouse = WinMouse()
    keyboard = WinKeyboard()
    _default_sys_encoding = sys.stdout.encoding or sys.getdefaultencoding()

    class _EnumWindowsCallback(object):

        last_handle = None

        @classmethod
        def callback(cls, handle, wildcard):
            wildcard = ctypes.cast(wildcard, ctypes.c_wchar_p).value

            wildcard = WinUtils.replace_inappropriate_symbols(wildcard)
            length = ctypes.windll.user32.GetWindowTextLengthW(handle) + 1
            buff = ctypes.create_unicode_buffer(length)
            ctypes.windll.user32.GetWindowTextW(handle, buff, length)
            win_text = WinUtils.replace_inappropriate_symbols(buff.value)

            if re.match(wildcard, win_text):
                cls.last_handle = handle

            return True

    def get_object_by_coordinates(self, x, y):
        obj_point = ctypes.wintypes.POINT()
        obj_point.x = x
        obj_point.y = y
        i_accessible = ctypes.POINTER(comtypes.gen.Accessibility.IAccessible)()
        obj_child_id = comtypes.automation.VARIANT()
        ctypes.oledll.oleacc.AccessibleObjectFromPoint(
            obj_point,
            ctypes.byref(i_accessible),
            ctypes.byref(obj_child_id))

        return WinElement(i_accessible, obj_child_id.value or 0)

    def is_window_exists(self, obj_handle):
        try:
            self.get_window(obj_handle)
            return True
        except TooSaltyUISoupException:
            return False

    def get_window(self, obj_handle=None):
        if obj_handle in (0, None):
            obj_handle = ctypes.windll.user32.GetDesktopWindow()
        elif isinstance(obj_handle, basestring):
            obj_name = unicode(obj_handle)

            regex = WinUtils.convert_wildcard_to_regex(obj_name)

            enum_windows_proc = \
                ctypes.WINFUNCTYPE(ctypes.c_bool, ctypes.POINTER(ctypes.c_int),
                                   ctypes.POINTER(ctypes.c_int))
            self._EnumWindowsCallback.last_handle = None
            ctypes.windll.user32.EnumWindows(enum_windows_proc(
                self._EnumWindowsCallback.callback),
                ctypes.c_wchar_p(regex))

            obj_handle = self._EnumWindowsCallback.last_handle

            if not obj_handle:
                obj_name = obj_name.encode(self._default_sys_encoding,
                                           errors='ignore')
                raise TooSaltyUISoupException('Can\'t find window "%s".' %
                                              obj_name)

        try:
            return WinElement(obj_handle, 0)
        except:
            raise TooSaltyUISoupException(
                'Error when retrieving window with handle=%r' % obj_handle)

    def get_visible_window_list(self):
        result = self.get_window().findall(
            only_visible=True,
            name=lambda x: x,
            role_name=lambda x: x in ['frm', 'pane'],
            location=lambda x: 0 not in x[2:])

        return result

    def get_visible_object_list(self, window_name):
        window = self.get_window(window_name)
        objects = window.findall(
            only_visible=True,
            role_name=lambda x: x != 'frm',
            location=lambda x: 0 not in x[2:])

        return objects



#!/usr/bin/env python
#
#    Copyright (c) 2014 Oleksandr Iakovenko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'alex.jakovenko@gmail.com'


from Quartz import CoreGraphics as CG

from time import sleep

from ..interfaces.i_mouse import IMouse
from ..utils.mac_utils import MacUtils

if MacUtils.is_python_3():
    xrange = range


class MacMouse(IMouse):

    LEFT_BUTTON = u'b1c'
    RIGHT_BUTTON = u'b3c'
    _SUPPORTED_BUTTON_NAMES = [LEFT_BUTTON, RIGHT_BUTTON]

    _LEFT_BUTTON_CODES = [
        CG.kCGEventLeftMouseDown,
        CG.kCGEventLeftMouseDragged,
        CG.kCGEventLeftMouseUp]
    _RIGHT_BUTTON_CODES = [
        CG.kCGEventRightMouseDown,
        CG.kCGEventRightMouseDragged,
        CG.kCGEventRightMouseUp]

    def _compose_mouse_event_chain(self, name, press=True, release=False):
        """
        Composes chain of mouse events based on button name and action flags.

        :param str name: Mouse button name. Should be one of: 
        'b1c' - left button or 'b3c' - right button.
        :param bool press: flag indicating whether event should indicate
        button press.
        :param bool release: indicating whether event should indicate
        button release.
        """
        mouse_event_chain = []
        if name == self.LEFT_BUTTON:
            if press:
                mouse_event_chain.append(CG.kCGEventLeftMouseDown)
            if release:
                mouse_event_chain.append(CG.kCGEventLeftMouseUp)
        if name == self.RIGHT_BUTTON:
            if press:
                mouse_event_chain.append(CG.kCGEventRightMouseDown)
            if release:
                mouse_event_chain.append(CG.kCGEventRightMouseUp)

        return mouse_event_chain

    def _do_event(self, code, x, y):
        """
        Generates mouse event for a special coordinate.

        :param int code: mouse event code.
        :param int x: x coordinate.
        :param int y: y coordinate.
        """
        if code in self._LEFT_BUTTON_CODES:
            button = CG.kCGMouseButtonLeft
        elif code in self._RIGHT_BUTTON_CODES:
            button = CG.kCGMouseButtonRight
        else:
            button = CG.kCGMouseButtonCenter

        CG.CGEventPost(
            CG.kCGHIDEventTap,
            CG.CGEventCreateMouseEvent(None, code, (x, y), button)
        )

    def _do_events(self, codes, x, y):
        """
        Generates a sequence of mouse events for a special coordinate.

        :param list[int] codes: mouse event code.
        :param int x: x coordinate.
        :param int y: y coordinate.
        """
        for code in codes:
            self._do_event(code, x, y)

    def move(self, x, y, smooth=True):
        MacUtils.verify_xy_coordinates(x, y)

        old_x, old_y = self.get_position()

        for i in xrange(100):
            intermediate_x = old_x + (x - old_x) * (i + 1) / 100.0
            intermediate_y = old_y + (y - old_y) * (i + 1) / 100.0
            smooth and sleep(.01)

            self._do_event(CG.kCGEventMouseMoved, int(intermediate_x),
                           int(intermediate_y))

    def drag(self, x1, y1, x2, y2, smooth=True):
        MacUtils.verify_xy_coordinates(x1, y1)
        MacUtils.verify_xy_coordinates(x2, y2)

        self.press_button(x1, y1, self.LEFT_BUTTON)

        for i in xrange(100):
            x = x1 + (x2 - x1) * (i + 1) / 100.0
            y = y1 + (y2 - y1) * (i + 1) / 100.0
            smooth and sleep(.01)
            self._do_event(CG.kCGEventLeftMouseDragged, x, y)

        self.release_button(self.LEFT_BUTTON)

    def press_button(self, x, y, button_name=LEFT_BUTTON):
        MacUtils.verify_xy_coordinates(x, y)
        MacUtils.verify_mouse_button_name(button_name,
                                          self._SUPPORTED_BUTTON_NAMES)

        event_codes = self._compose_mouse_event_chain(
            button_name, press=True, release=False)
        self._do_events(event_codes, x, y)

    def release_button(self, button_name=LEFT_BUTTON):
        MacUtils.verify_mouse_button_name(button_name,
                                          self._SUPPORTED_BUTTON_NAMES)

        event_codes = self._compose_mouse_event_chain(
            button_name, press=False, release=True)
        self._do_events(event_codes, 0, 0)

    def click(self, x, y, button_name=LEFT_BUTTON):
        MacUtils.verify_xy_coordinates(x, y)
        MacUtils.verify_mouse_button_name(button_name,
                                          self._SUPPORTED_BUTTON_NAMES)

        event_codes = self._compose_mouse_event_chain(
            button_name, press=True, release=True)
        self._do_events(event_codes, x, y)

    def double_click(self, x, y, button_name=LEFT_BUTTON):
        MacUtils.verify_xy_coordinates(x, y)
        MacUtils.verify_mouse_button_name(button_name,
                                          self._SUPPORTED_BUTTON_NAMES)

        if button_name == self.LEFT_BUTTON:
            button = CG.kCGMouseButtonLeft
            down = CG.kCGEventLeftMouseDown
            up = CG.kCGEventLeftMouseUp
        if button_name == self.RIGHT_BUTTON:
            button = CG.kCGMouseButtonRight
            down = CG.kCGEventRightMouseDown
            up = CG.kCGEventRightMouseUp

        # http://www.codeitive.com/0iJqgkejVj/performing-a-double-click-using-cgeventcreatemouseevent.html
        event = CG.CGEventCreateMouseEvent(None, down, (x, y), button)
        CG.CGEventPost(CG.kCGHIDEventTap, event)
        CG.CGEventSetType(event, up)
        CG.CGEventPost(CG.kCGHIDEventTap, event)

        CG.CGEventSetIntegerValueField(event, CG.kCGMouseEventClickState, 2)

        CG.CGEventSetType(event, down)
        CG.CGEventPost(CG.kCGHIDEventTap, event)
        CG.CGEventSetType(event, up)
        CG.CGEventPost(CG.kCGHIDEventTap, event)

    def get_position(self):
        position = CG.CGEventGetLocation(CG.CGEventCreate(None))
        return int(position.x), int(position.y)



#!/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'


from platform import system

uisoup = None


class TooSaltyUISoupException(Exception):
    pass

if system() == 'Windows':
    from uisoup.win_soup import WinSoup
    uisoup = WinSoup()
elif system() == 'Darwin':
    from uisoup.mac_soup import MacSoup
    uisoup = MacSoup()
else:
    raise TooSaltyUISoupException('We are sorry but we don\'t have UISoup '
                                  'implementation for "%s" OS.' % system())



# !/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'


import struct
from AppKit import NSAppleScript
from Carbon import AppleEvents

from ..utils.common import CommonUtils
from .. import TooSaltyUISoupException

if CommonUtils.is_python_3():
    basestring = str


class AppleEventDescriptor(object):

    @classmethod
    def _get_aeKeyword(cls, four_char_code):
        """
        Gets aeKeyword from four character event code.

        Arguments:
            - four_char_code: str, four character event code.

        Return:
            - integer aeKeyword.
        """

        return struct.unpack('>I', four_char_code)[0]

    @classmethod
    def _get_four_char_code(cls, ae_keyword):
        """
        Gets four char event code from aeKeyword.

        Arguments:
            - ae_keyword: integer, aeKeyword.

        Return:
            - string four char code.
        """

        return struct.pack('>I', ae_keyword)

    def __init__(self, event_descriptor):
        """
        Constructor.

        Arguments:
            - event_descriptor: NSAppleEventDescriptor, instance.
        """

        self._event_descriptor = event_descriptor

    def __iter__(self):
        """Iterate all nested elements"""

        if not self.class_name:
            i = 1
            while self._event_descriptor.descriptorAtIndex_(i):
                yield AppleEventDescriptor(
                    self._event_descriptor.descriptorAtIndex_(i))
                i += 1

        raise StopIteration()

    @property
    def form_(self):
        """
        Property for element "from" field.
        """

        from_ = self._event_descriptor.descriptorForKeyword_(
            self._get_aeKeyword(AppleEvents.keyAEKeyForm))

        result = AppleEventDescriptor(from_).string_value if from_ else None

        return result

    @property
    def class_name(self):
        """
        Property for element class name.
        """

        result = None

        if self._event_descriptor.typeCodeValue():
            ae_keyword = \
                self._event_descriptor.descriptorForKeyword_(
                    self._get_aeKeyword(
                        AppleEvents.keyAEDesiredClass))
            if ae_keyword:
                result = \
                    self._get_four_char_code(int(ae_keyword.typeCodeValue()))

        return result

    @property
    def class_id(self):
        """
        Property for element class id.
        """

        return (self.seld_.string_value or '').replace('\\', '\\\\')

    @property
    def seld_(self):
        """
        Property for "seld" field.
        """

        seld_ = self._event_descriptor.descriptorForKeyword_(
            self._get_aeKeyword(AppleEvents.keyAEKeyData))

        result = AppleEventDescriptor(seld_) if seld_ else None

        return result

    @property
    def from_(self):
        """
        Property for "from" field.
        """

        from_ = self._event_descriptor.descriptorForKeyword_(
            self._get_aeKeyword(AppleEvents.keyAEContainer))

        result = AppleEventDescriptor(from_) if from_ else None

        return result

    @property
    def string_value(self):
        """
        Property for string value.
        """

        return self._event_descriptor.stringValue()

    @property
    def applescript_specifier(self):
        """
        Property for applescript specifier.
        """

        if self.class_id:
            class_id = \
                self.class_id if self.form_ == AppleEvents.kFAIndexParam else \
                u'"%s"' % self.class_id.replace('"', '\\"')
            specifier = u'«class %s» %s' % (self.class_name, class_id)
        else:
            specifier = u'every «class %s»' % self.class_name

        if self.from_.class_name:
            # Sometimes applescript returns menu bar as a child of
            # window (cwin) but this is wrong and parent of menu bar should
            # be application (pcap) so we need to skip one parent (cwin) in
            # this case.
            parent_element = self.from_
            if self.class_name == 'mbar' and\
                    self.from_.class_name == AppleEvents.cWindow:
                parent_element = parent_element.from_
            specifier = '%s of %s' % (specifier,
                                      parent_element.applescript_specifier)
        else:
            specifier = '%s of application "System Events"' % specifier

        return specifier


class MacUtils(CommonUtils):

    @classmethod
    def execute_applescript_command(cls, cmd):
        """
        Executes applescript command.

        :param str | list[str] cmd: command or commands that should be
        executed.
        :rtype: AppleEventDescriptor
        :return: AppleEventDescriptor with result of executed command.
        """
        cmd = '\n'.join([cmd] if isinstance(cmd, basestring) else cmd)

        script = NSAppleScript.alloc().initWithSource_(cmd)
        result = script.executeAndReturnError_(None)

        if not result[0]:
            error_message = 'Error when executing applescript command: %s' %\
                            result[1]['NSAppleScriptErrorMessage']
            raise TooSaltyUISoupException(error_message.encode('utf-8',
                                                               'ignore'))

        return AppleEventDescriptor(result[0])

    class ApplescriptExecutor(object):

        @classmethod
        def get_frontmost_window_name(cls):
            """
            Gets front window name.

            :rtype: str
            :return: combined name (window name + process name).
            """
            cmd = ['tell application "System Events" to tell process (name of first application process whose frontmost is true)',
                   '  return (1st window whose value of attribute "AXMain" is true)',
                   'end tell']

            event_descriptor = MacUtils.execute_applescript_command(cmd)

            return event_descriptor.seld_.string_value + \
                event_descriptor.from_.seld_.string_value

        @classmethod
        def get_apple_event_descriptor(cls, obj_selector, process_name):
            """
            Gets apple event descriptor.

            :param str obj_selector: object selector.
            :param str process_name: name of process.
            
            :rtype: AppleEventDescriptor
            :return: apple event descriptor.
            """
            cmd = ['tell application "System Events" to tell process "%s"' % process_name,
                   '  set visible to true',
                   '  return %s' % obj_selector,
                   'end tell']

            return MacUtils.execute_applescript_command(cmd)

        @classmethod
        def get_children_elements(cls, obj_selector, layer_num, process_name):
            """
            Gets all direct children elements.

            :param str obj_selector: object selector.
            :param str layer_num: layer number, i.e. main window will be 
            layer 0.
            :param str process_name: name of process.
            :rtype: tuple[dict]
            :return: Tuple that contains dict of element selectors and
            elements layer number.
            """
            cmd = ['tell application "System Events" to tell process "%s"' % process_name,
                   '  set visible to true',
                   '  set uiElement to %s' % obj_selector,
                   '  set layer to %s' % layer_num,
                   '  if uiElement = null then',
                   '    set layer to 0',
                   '    set collectedElements to {UI elements of front window, layer}',
                   '  else',
                   '    set layer to layer + 1',
                   '    set collectedElements to {null, layer}',
                   '    if name of attributes of uiElement contains "AXChildren" then',
                   '        set collectedElements to {value of attribute "AXChildren" of uiElement, layer}',
                   '    end if',
                   '  end if',
                   'end tell',
                   'return collectedElements']

            event_descriptors_list = \
                list(MacUtils.execute_applescript_command(cmd))

            elements = [{'selector': el.applescript_specifier,
                         'class_id': el.class_id} for el in
                        list(event_descriptors_list[0])]

            return elements, int(event_descriptors_list[1].string_value)

        @classmethod
        def get_element_properties(cls, obj_selector, process_name):
            """
            Gets all element properties.

            :param str obj_selector: object selector.
            :param str process_name: name of process.
            :rtype: dict
            :return: Dict with element properties.
            """
            cmd = ['tell application "System Events" to tell application process "%s"' % process_name,
                   '  set visible to true',
                   '  set res to {}',
                   '  repeat with attr in attributes of %s' % obj_selector,
                   '    try',
                   '      set res to res & {{name of attr, value of attr}}',
                   '    end try',
                   '  end repeat',
                   '  return res',
                   'end tell']

            event_descriptors_list = list(
                MacUtils.execute_applescript_command(cmd))

            # Unpacking properties to dict.
            el_properties = dict()
            for prop in event_descriptors_list:
                prop = list(prop)
                prop_name = prop[0].string_value
                prop_value = [e.string_value for e in list(prop[1])] if \
                    list(prop[1]) else prop[1].string_value

                el_properties[prop_name] = prop_value

            return el_properties

        @classmethod
        def set_element_attribute_value(cls, obj_selector, attribute_name,
                                        value, process_name,
                                        string_value=True):
            """
            Sets element attribute.

            :param str obj_selector: object selector.
            :param str attribute_name: name of attribute.
            :param str value: name of value.
            :param str process_name: name of process.
            :param bool string_value: indicates will be value wrapped in 
            brackets.
            :rtype: bool
            :return: indicator whether was operation successful or not.
            """
            value = '"%s"' % value if string_value else value
            cmd = ['tell application "System Events" to tell application process "%s"' % process_name,
                   '  set value of attribute "%s" of %s to %s' % (attribute_name, obj_selector, value),
                   'end tell']

            try:
                MacUtils.execute_applescript_command(cmd)
                result = True
            except TooSaltyUISoupException:
                result = False

            return result

        @classmethod
        def get_axunknown_windows(cls, process_name):
            """
            Gets AXUnknown windows by given process name.

            :param str process_name: name of process.
            :rtype: list
            :return: list of AppleEventDescriptor elements.
            """

            cmd = ['tell application "System Events" to tell process "%s"' % process_name,
                   '  set visible to true',
                   '  set unknownWindows to {}',
                   '  repeat with e in UI elements',
                   '    if value of attribute "AXRole" of e is equal to "AXUnknown" then',
                   '      set unknownWindows to unknownWindows & {e}',
                   '    end if',
                   '  end repeat',
                   '  return unknownWindows',
                   'end tell']

            return list(MacUtils.execute_applescript_command(cmd))

        @classmethod
        def get_axdialog_windows(cls, process_name):
            """
            Gets AXDialog windows by given process name.

            :param str process_name: name of process.
            :rtype: list
            :return: list of AppleEventDescriptor elements.
            """

            cmd = ['tell application "System Events" to tell process "%s"' % process_name,
                   '  set visible to true',
                   '  set dialogWindows to {}',
                   '  repeat with e in UI elements',
                   '    if value of attribute "AXRole" of e is equal to "AXWindow" and value of attribute "AXSubrole" of e is equal to "AXDialog" then',
                   '      set dialogWindows to dialogWindows & {e}',
                   '    end if',
                   '  end repeat',
                   '  return dialogWindows',
                   'end tell']

            return list(MacUtils.execute_applescript_command(cmd))



#!/usr/bin/env python
#
#    Copyright (c) 2014 Oleksandr Iakovenko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'alex.jakovenko@gmail.com'

from Quartz import CoreGraphics as CG

from time import sleep

from ..interfaces.i_keyboard import Key, IKeyboard


class MacKeyboard(IKeyboard):

    class _KeyCodes(object):
        """
        Holder for Macintosh keyboard codes stored as Keys.
        """
        LEFT_ALT = Key(0x3A)  # Left ALT key
        RIGHT_ALT = Key(0x3D)  # Right ALT key
        LEFT_SHIFT = Key(0x38)  # Left SHIFT key
        RIGHT_SHIFT = Key(0x3C)  # Right SHIFT key
        LEFT_CONTROL = Key(0x3B)  # Left CONTROL key
        RIGHT_CONTROL = Key(0x3E)  # Right CONTROL key
        LEFT_COMMAND = Key(0x37)  # Left COMMAND key
        RIGHT_COMMAND = Key(0x36)  # Right COMMAND key
        BACKSPACE = Key(0x33)  # BACKSPACE key
        TAB = Key(0x30)  # TAB key
        CLEAR = Key(0x47)  # CLEAR key
        RETURN = Key(0x24)  # ENTER key
        SHIFT = LEFT_SHIFT  # SHIFT key
        CONTROL = LEFT_CONTROL  # CTRL key
        ALT = LEFT_ALT  # ALT key
        CAPS_LOCK = Key(0x39)  # CAPS LOCK key
        ESCAPE = Key(0x35)  # ESC key
        SPACE = Key(0x31)  # SPACEBAR
        PAGE_UP = Key(0x74)  # PAGE UP key
        PAGE_DOWN = Key(0x79)  # PAGE DOWN key
        END = Key(0x77)  # END key
        HOME = Key(0x73)  # HOME key
        LEFT = Key(0x7B)  # LEFT ARROW key
        UP = Key(0x7E)  # UP ARROW key
        RIGHT = Key(0x7C)  # RIGHT ARROW key
        DOWN = Key(0x7D)  # DOWN ARROW key
        INSERT = Key(0x72)  # INS key
        DELETE = Key(0x75)  # DEL key
        KEY_0 = Key(0x1D)  # 0 key
        KEY_1 = Key(0x12)  # 1 key
        KEY_2 = Key(0x13)  # 2 key
        KEY_3 = Key(0x14)  # 3 key
        KEY_4 = Key(0x15)  # 4 key
        KEY_5 = Key(0x17)  # 5 key
        KEY_6 = Key(0x16)  # 6 key
        KEY_7 = Key(0x1A)  # 7 key
        KEY_8 = Key(0x1C)  # 8 key
        KEY_9 = Key(0x19)  # 9 key
        KEY_A = Key(0x00)  # A key
        KEY_B = Key(0x0B)  # B key
        KEY_C = Key(0x08)  # C key
        KEY_D = Key(0x02)  # D key
        KEY_E = Key(0x0E)  # E key
        KEY_F = Key(0x03)  # F key
        KEY_G = Key(0x05)  # G key
        KEY_H = Key(0x04)  # H key
        KEY_I = Key(0x22)  # I key
        KEY_J = Key(0x26)  # J key
        KEY_K = Key(0x28)  # K key
        KEY_L = Key(0x25)  # L key
        KEY_M = Key(0x2E)  # M key
        KEY_N = Key(0x2D)  # N key
        KEY_O = Key(0x1F)  # O key
        KEY_P = Key(0x23)  # P key
        KEY_Q = Key(0x0C)  # Q key
        KEY_R = Key(0x0F)  # R key
        KEY_S = Key(0x01)  # S key
        KEY_T = Key(0x11)  # T key
        KEY_U = Key(0x20)  # U key
        KEY_V = Key(0x09)  # V key
        KEY_W = Key(0x0D)  # W key
        KEY_X = Key(0x07)  # X key
        KEY_Y = Key(0x10)  # Y key
        KEY_Z = Key(0x06)  # Z key
        NUMPAD0 = Key(0x52)  # Numeric keypad 0 key
        NUMPAD1 = Key(0x53)  # Numeric keypad 1 key
        NUMPAD2 = Key(0x54)  # Numeric keypad 2 key
        NUMPAD3 = Key(0x55)  # Numeric keypad 3 key
        NUMPAD4 = Key(0x56)  # Numeric keypad 4 key
        NUMPAD5 = Key(0x57)  # Numeric keypad 5 key
        NUMPAD6 = Key(0x58)  # Numeric keypad 6 key
        NUMPAD7 = Key(0x59)  # Numeric keypad 7 key
        NUMPAD8 = Key(0x5B)  # Numeric keypad 8 key
        NUMPAD9 = Key(0x5C)  # Numeric keypad 9 key
        MULTIPLY = Key(0x43)  # Multiply key
        ADD = Key(0x45)  # Add key
        SUBTRACT = Key(0x4E)  # Subtract key
        DECIMAL = Key(0x41)  # Decimal key
        DIVIDE = Key(0x4B)  # Divide key
        F1 = Key(0x7A)  # F1 key
        F2 = Key(0x78)  # F2 key
        F3 = Key(0x63)  # F3 key
        F4 = Key(0x76)  # F4 key
        F5 = Key(0x60)  # F5 key
        F6 = Key(0x61)  # F6 key
        F7 = Key(0x62)  # F7 key
        F8 = Key(0x64)  # F8 key
        F9 = Key(0x65)  # F9 key
        F10 = Key(0x6D)  # F10 key
        F11 = Key(0x67)  # F11 key
        F12 = Key(0x6F)  # F12 key
        OEM_1 = Key(0x29)  # For the US standard keyboard, the ';:' key
        OEM_PLUS = Key(0x18)  # For any country/region, the '+' key
        OEM_COMMA = Key(0x2B)  # For any country/region, the ',' key
        OEM_MINUS = Key(0x1B)  # For any country/region, the '-' key
        OEM_PERIOD = Key(0x2F)  # For any country/region, the '.' key
        OEM_2 = Key(0x2C)  # For the US standard keyboard, the '/?' key
        OEM_3 = Key(0x32)  # For the US standard keyboard, the '`~' key
        OEM_4 = Key(0x21)  # For the US standard keyboard, the '[{' key
        OEM_5 = Key(0x2A)  # For the US standard keyboard, the '\|' key
        OEM_6 = Key(0x1E)  # For the US standard keyboard, the ']}' key
        OEM_7 = Key(0x27)  # For the US standard keyboard, the ''"' key

    codes = _KeyCodes

    def press_key(self, hex_key_code):
        """
        Presses (and releases) key specified by a hex code.

        :param int hex_key_code: hexadecimal code for a key to be pressed.
        """
        self.press_key_and_hold(hex_key_code)
        self.release_key(hex_key_code)

    def press_key_and_hold(self, hex_key_code):
        """
        Presses (and holds) key specified by a hex code.

        :param int hex_key_code: hexadecimal code for a key to be pressed.
        """
        CG.CGEventPost(
            CG.kCGHIDEventTap,
            CG.CGEventCreateKeyboardEvent(None, hex_key_code, True))

    def release_key(self, hex_key_code):
        """
        Releases key specified by a hex code.

        :param int hex_key_code: hexadecimal code for a key to be pressed.
        """
        CG.CGEventPost(
            CG.kCGHIDEventTap,
            CG.CGEventCreateKeyboardEvent(None, hex_key_code, False))

    def send(self, *args, **kwargs):
        """
        Send key events as specified by Keys.

        If Key contains children Keys they will be recursively
        processed with current Key code pressed as a modifier key.

        :param args: Keys to send.
        """
        delay = kwargs.get('delay', 0)

        for key in args:
            if key.children:
                self.press_key_and_hold(key.code)
                self._wait_for_key_combo_to_be_processed()
                self.send(*key.children)
                self.release_key(key.code)
            else:
                self.press_key(key.code)
            self._wait_for_key_combo_to_be_processed()
            sleep(delay)

    def _wait_for_key_combo_to_be_processed(self):
        # For key combinations timeout is needed to be processed.
        # This method is expressive shortcut to be used where needed.
        sleep(.05)



#!/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

from abc import ABCMeta, abstractmethod, abstractproperty


class IMouse(object):
    """
    Class to simulate mouse activities.
    """

    __metaclass__ = ABCMeta

    @abstractproperty
    def LEFT_BUTTON(self):
        """
        Constant for left mouse button.
        """

    @abstractproperty
    def RIGHT_BUTTON(self):
        """
        Constant for right mouse button.
        """

    @abstractmethod
    def move(self, x, y, smooth=True):
        """
        Move the mouse to the specified coordinates.

        :param int x: x coordinate.
        :param int y: y coordinate.
        :param bool smooth: indicates is it needed to simulate smooth movement.
        """

    @abstractmethod
    def drag(self, x1, y1, x2, y2, smooth=True):
        """
        Drags the mouse to the specified coordinates.

        :param int x1: x start coordinate.
        :param int y1: y start coordinate.
        :param int x2: x target coordinate.
        :param int y2: y target coordinate.
        :param bool smooth: indicates is it needed to simulate smooth movement.
        """

    @abstractmethod
    def press_button(self, x, y, button_name=LEFT_BUTTON):
        """
        Presses mouse button as dictated by coordinates and button name.

        :param int x: x coordinate to press mouse at.
        :param int y: y coordinate to press mouse at.
        :param str button_name: mouse button name. Should be one
        of: 'b1c' - left button or 'b3c' - right button.
        """

    @abstractmethod
    def release_button(self, button_name=LEFT_BUTTON):
        """
        Releases mouse button by button name.

        :param str button_name: mouse button name. Should be one
        of: 'b1c' - left button or 'b3c' - right button.
        """

    @abstractmethod
    def click(self, x, y, button_name=LEFT_BUTTON):
        """
        Clicks as dictated by coordinates and button name.

        :param int x: x coordinate to click mouse at.
        :param int y: y coordinate to click mouse at.
        :param str button_name: mouse button name. Should be one
        of: 'b1c' - left button or 'b3c' - right button.
        """

    @abstractmethod
    def double_click(self, x, y, button_name=LEFT_BUTTON):
        """
        Double-clicks as dictated by coordinates and button name.

        :param int x: x coordinate to double-click at.
        :param int y: y coordinate to double-click at.
        :param str button_name: mouse button name. Should be one
        of: 'b1c' - left button or 'b3c' - right button.
        """

    @abstractmethod
    def get_position(self):
        """
        Returns current mouse cursor position.

        :rtype: tuple[int, int]
        :return: x and y coordinates of current mouse cursor position.
        """



# !/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

import sys
import re

from Quartz import CoreGraphics as CG

from ..interfaces.i_soup import ISoup
from ..utils.mac_utils import MacUtils
from .element import MacElement
from .mouse import MacMouse
from .keyboard import MacKeyboard
from .. import TooSaltyUISoupException

if MacUtils.is_python_3():
    unicode = str


class MacSoup(ISoup):

    mouse = MacMouse()
    keyboard = MacKeyboard()
    _default_sys_encoding = sys.stdout.encoding or sys.getdefaultencoding()

    def get_object_by_coordinates(self, x, y):
        result = None

        try:
            window_handle = \
                MacUtils.ApplescriptExecutor.get_frontmost_window_name()

            window = self.get_window(window_handle)

            # Sorting by layer from big to small.
            if MacUtils.is_python_3():
                sorted_objects = \
                    sorted(window.findall(), key=lambda x: x._layer_num,
                           reverse=True)
            else:
                sorted_objects = \
                    sorted(window.findall(),
                           lambda x, y: y._layer_num - x._layer_num)

            cur_x, cur_y = self.mouse.get_position()
            for element in sorted_objects:
                x, y, w, h = element.acc_location
                if x <= cur_x < x + w and y <= cur_y < y + h:
                    result = element
                    break
        except:
            pass

        return result

    def is_window_exists(self, obj_handle):
        try:
            self.get_window(obj_handle)
            return True
        except TooSaltyUISoupException:
            return False

    def get_window(self, obj_handle=None):
        filters = CG.kCGWindowListOptionOnScreenOnly | \
            CG.kCGWindowListExcludeDesktopElements * bool(obj_handle)

        obj_name = obj_handle if obj_handle else u'DesktopWindow Server'
        obj_name = \
            obj_name if type(obj_name) == unicode else obj_name.decode('utf-8')

        regex = MacUtils.convert_wildcard_to_regex(obj_name)

        win_list = CG.CGWindowListCopyWindowInfo(filters, CG.kCGNullWindowID)

        window = filter(lambda x:
                        re.match(MacUtils.replace_inappropriate_symbols(regex),
                                 MacUtils.replace_inappropriate_symbols(
                                     x.get('kCGWindowName', '')) +
                                 x.get('kCGWindowOwnerName', ''),
                                 re.IGNORECASE)
                        if x.get('kCGWindowName', '') else False,
                        win_list)

        window = window[0] if window else window
        if not window:
            obj_name = obj_name.encode(self._default_sys_encoding,
                                       errors='ignore')
            raise TooSaltyUISoupException('Can\'t find window "%s".' %
                                          obj_name)

        process_name = window['kCGWindowOwnerName']
        window_name = window['kCGWindowName']
        process_id = int(window['kCGWindowOwnerPID'])
        # Escape double quotes in window name.
        window_name = window_name.replace('"', '\\"')

        selector = \
            MacUtils.ApplescriptExecutor.get_apple_event_descriptor(
                'window "%s"' % window_name,
                process_name).applescript_specifier
        selector = \
            selector if type(selector) == unicode else selector.decode('utf-8')

        return MacElement(selector, 0, process_name, process_id)

    def get_visible_window_list(self):
        win_list = CG.CGWindowListCopyWindowInfo(
            CG.kCGWindowListOptionOnScreenOnly |
            CG.kCGWindowListExcludeDesktopElements,
            CG.kCGNullWindowID)

        win_names = \
            [w.get('kCGWindowName', '') + w.get('kCGWindowOwnerName', '') for
             w in win_list if w.get('kCGWindowName', '') and 0 not in
             [int(w.get('kCGWindowBounds', 0)['Height']),
              int(w.get('kCGWindowBounds', 0)['Width'])]]

        windows = list()
        for win_name in win_names:
            try:
                windows.append(self.get_window(win_name))
            except TooSaltyUISoupException:
                continue

        return windows

    def get_visible_object_list(self, window_name):
        window = self.get_window(window_name)
        objects = window.findall(
            only_visible=True,
            location=lambda x: 0 not in x[2:])

        return objects



# !/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

from ..interfaces.i_element import IElement
from ..utils.mac_utils import MacUtils
from .. import TooSaltyUISoupException
from .mouse import MacMouse


class MacElement(IElement):

    _acc_role_name_map = {
        'AXWindow': u'frm',
        'AXTextArea': u'txt',
        'AXTextField': u'txt',
        'AXButton': u'btn',
        'AXStaticText': u'lbl',
        'AXRadioButton': u'rbtn',
        'AXSlider': u'sldr',
        'AXCell': u'tblc',
        'AXImage': u'img',
        'AXToolbar': u'tbar',
        'AXScrollBar': u'scbr',
        'AXMenuItem': u'mnu',
        'AXMenu': u'mnu',
        'AXMenuBar': u'mnu',
        'AXMenuBarItem': u'mnu',
        'AXCheckBox': u'chk',
        'AXTabGroup': u'ptl',
        'AXList': u'lst',
        'AXMenuButton': u'cbo',
        'AXRow': u'tblc',
        'AXColumn': u'col',
        'AXTable': u'tbl',
        'AXScrollArea': u'sar',
        'AXOutline': u'otl',
        'AXValueIndicator': u'val',
        'AXDisclosureTriangle': u'dct',
        'AXGroup': u'grp',
        'AXPopUpButton': u'pubtn',
        'AXApplication': u'app',
        'AXDocItem': u'doc',
        'AXHeading': u'tch',
        'AXGenericElement': u'gen',
        'AXLink': u'lnk'
    }

    _mouse = MacMouse()

    def __init__(self, obj_selector, layer_num, process_name, process_id,
                 class_id=None):
        """
        Constructor.

        :param str obj_selector: object selector.
        :param int layer_num: layer number. I.e. main window will be layer 0.
        :param str process_name: process name.
        :param str process_id: process id.
        :param str class_id: element class identifier.
        """
        self._object_selector = obj_selector
        self._layer_num = layer_num
        self._proc_id = process_id
        self._proc_name = process_name
        self._class_id = class_id
        self._cached_children = set()
        self._cached_properties = None

    @property
    def _properties(self):
        """
        Property for element properties.
        """
        if not self._cached_properties:
            self._cached_properties = \
                MacUtils.ApplescriptExecutor.get_element_properties(
                    self._object_selector, self._proc_name)

        return self._cached_properties

    @property
    def _role(self):
        """
        Property for element role.
        """
        return self._properties.get('AXRole', None)

    def _find_windows_by_same_proc(self):
        """
        Find window by same process id.

        :rtype: list[uisoup.interfaces.i_element.IElement]
        :return: list of windows.
        """
        axunknown_windows = \
            MacUtils.ApplescriptExecutor.get_axunknown_windows(self._proc_name)
        axdialog_windows = \
            MacUtils.ApplescriptExecutor.get_axdialog_windows(self._proc_name)

        mac_elements = \
            [MacElement(element.applescript_specifier, 1, self._proc_name,
                        self._proc_id, element.class_id) for element in
             axunknown_windows + axdialog_windows]

        return filter(lambda x: x.acc_child_count, mac_elements)

    def click(self, x_offset=0, y_offset=0):
        x, y, w, h = self.acc_location
        x += x_offset if x_offset is not None else w / 2
        y += y_offset if y_offset is not None else h / 2

        self._mouse.click(x, y)
        self._cached_properties = None

    def right_click(self, x_offset=0, y_offset=0):
        x, y, w, h = self.acc_location
        x += x_offset if x_offset is not None else w / 2
        y += y_offset if y_offset is not None else h / 2

        self._mouse.click(x, y, self._mouse.RIGHT_BUTTON)
        self._cached_properties = None

    def double_click(self, x_offset=0, y_offset=0):
        x, y, w, h = self.acc_location
        x += x_offset if x_offset is not None else w / 2
        y += y_offset if y_offset is not None else h / 2

        self._mouse.double_click(x, y)
        self._cached_properties = None

    def drag_to(self, x, y, x_offset=None, y_offset=None, smooth=True):
        el_x, el_y, el_w, el_h = self.acc_location
        el_x += x_offset if x_offset is not None else el_w / 2
        el_y += y_offset if y_offset is not None else el_h / 2

        self._mouse.drag(el_x, el_y, x, y, smooth)
        self._cached_properties = None

    @property
    def proc_id(self):
        return self._proc_id

    @property
    def is_top_level_window(self):
        return self.acc_parent_count == 0

    @property
    def is_selected(self):
        result = False
        if self.acc_role_name == self._acc_role_name_map['AXRadioButton'] and \
                self._properties.get('AXValue', 'false') == 'true':
            result = True

        return result

    @property
    def is_checked(self):
        result = False
        if self.acc_role_name == self._acc_role_name_map['AXCheckBox'] and \
                self._properties.get('AXValue', 'false') == 'true':
            result = True

        return result

    @property
    def is_visible(self):
        return True  # We can't work with invisible elements under Mac OS.

    @property
    def is_enabled(self):
        return bool(self._properties.get('AXEnabled', False))

    @property
    def acc_parent_count(self):
        return self._layer_num

    @property
    def acc_child_count(self):
        children_elements = MacUtils.ApplescriptExecutor.get_children_elements(
            self._object_selector, self.acc_parent_count, self._proc_name)

        return len(children_elements[0])

    @property
    def acc_name(self):
        result = self._properties.get('AXDescription') or \
            self._properties.get('AXTitle')

        if not result:
            if self.acc_role_name == self._acc_role_name_map['AXTextField']:
                result = self._class_id
            else:
                result = self._properties.get('AXValue') or self._class_id

        return MacUtils.replace_inappropriate_symbols(result or '')

    def set_focus(self):
        MacUtils.ApplescriptExecutor.set_element_attribute_value(
            self._object_selector, 'AXFocused', 'true', self._proc_name, False)

    @property
    def acc_c_name(self):
        return self.acc_role_name + self.acc_name if self.acc_name else ''

    @property
    def acc_location(self):
        x, y = self._properties.get('AXPosition', [0, 0])
        w, h = self._properties.get('AXSize', [0, 0])

        return map(int, [x, y, w, h])

    @property
    def acc_value(self):
        return self._properties.get('AXValue', None)

    def set_value(self, value):
        MacUtils.ApplescriptExecutor.set_element_attribute_value(
            self._object_selector, 'AXValue', value, self._proc_name)
        self._cached_properties = None

    @property
    def acc_description(self):
        return self._properties.get('AXDescription', None)

    @property
    def acc_parent(self):
        result = None
        event_descriptor = \
            MacUtils.ApplescriptExecutor.get_apple_event_descriptor(
                self._object_selector, self._proc_name)
        if self.acc_parent_count > 0 and event_descriptor.from_:
            result = \
                MacElement(event_descriptor.from_.applescript_specifier,
                           self.acc_parent_count - 1,
                           self._proc_name,
                           self.proc_id,
                           event_descriptor.class_id)

        return result

    @property
    def acc_selection(self):
        return self._properties.get('AXSelectedText', None)

    @property
    def acc_focused_element(self):
        childs = self.findall()

        result = None
        for element in childs:
            if self._properties.get('AXFocused', 'false') == 'true':
                result = element
                break

        return result

    @property
    def acc_role_name(self):
        return self._acc_role_name_map.get(self._role, 'unknown')

    def __iter__(self):
        children_elements = MacUtils.ApplescriptExecutor.get_children_elements(
            self._object_selector, self._layer_num, self._proc_name)

        children = children_elements[0]
        layer_number = children_elements[1]

        if not len(children):
            raise StopIteration()

        for element in children:
            yield MacElement(element['selector'], layer_number,
                             self._proc_name, self.proc_id,
                             element['class_id'])

    def __findcacheiter(self, only_visible, **kwargs):
        """
        Find child element in the cache.

        :param bool only_visible: flag that indicates will we search only 
        visible.
        :rtype: uisoup.interfaces.i_element.IElement
        :return: yield found element.
        """
        for obj_element in self._cached_children:
            if obj_element._match(only_visible, **kwargs):
                yield obj_element

    def _finditer(self, only_visible, **kwargs):
        """
        Find child element.

        :param bool only_visible: flag that indicates will we search only 
        visible.
        :rtype: uisoup.interfaces.i_element.IElement
        :return: yield found element.
        """
        lst_queue = list(self)

        if self.is_top_level_window:
            lst_queue.extend(self._find_windows_by_same_proc())

        while lst_queue:
            obj_element = lst_queue.pop(0)
            self._cached_children.add(obj_element)

            if obj_element._match(only_visible, **kwargs):
                yield obj_element

            if obj_element.acc_child_count:
                childs = [el for el in list(obj_element)]
                lst_queue[:0] = childs

    def find(self, only_visible=True, **kwargs):
        try:
            iter_ = next(self.__findcacheiter(only_visible, **kwargs)) if \
                MacUtils.is_python_3() else self.__findcacheiter(
                only_visible, **kwargs).next()
            return iter_
        except StopIteration:
            try:
                iter_ = next(self._finditer(only_visible, **kwargs)) if \
                    MacUtils.is_python_3() else self._finditer(
                    only_visible, **kwargs).next()
                return iter_
            except StopIteration:
                items_ = kwargs.items() if MacUtils.is_python_3() else \
                    kwargs.iteritems()
                attrs = ['%s=%s' % (k, v) for k, v in items_]
                raise TooSaltyUISoupException(
                    'Can\'t find object with attributes "%s".' %
                    '; '.join(attrs))

    def findall(self, only_visible=True, **kwargs):
        result = self._finditer(only_visible, **kwargs)
        if result:
            result = list(result)

        return result

    def is_object_exists(self, **kwargs):
        try:
            self.find(**kwargs)
            return True
        except TooSaltyUISoupException:
            return False



#!/usr/bin/env python

from os import path

from pip.req import parse_requirements
from setuptools import setup


def package_env(file_name, strict=False):
    file_path = path.join(path.dirname(__file__), file_name)
    if path.exists(file_path) or strict:
        return open(file_path).read()
    else:
        return ''

if __name__ == '__main__':
    setup(
        name='UISoup',
        version='2.5.1',
        description='Cross Platform GUI Test Automation tool.',
        long_description=package_env('README.rst'),
        author='Max Beloborodko',
        author_email='f1ashhimself@gmail.com',
        packages=['uisoup'],
        include_package_data=True,
        install_requires=[str(ir.req) for ir in
                          parse_requirements('requirements.txt',
                                             session=False)],
        zip_safe=False,
        entry_points={
            'console_scripts': [
                'ui-inspector = uisoup.ui_inspector:main'
            ]
        }
    )
